clear;
close all;
d2r = pi/180;
r2d = 180/pi;
water_density = 1025; % [kg/m^3]
g = 9.81;
Ts = 0.001; % Sample time (if fixed)

U_0 = 17; % Operating speed in surge [m/s]
z_d = -2.5; % Operating position in heave [m/s]

%% Workspace input for model data

boat.length = 24;   % [m]
boat.length_width_ratio = 3;
boat.height_width_ratio = 3/4;
boat.mass = 60000;  % [kg]
boat.T_f_ratio = 2/3;
boat.delta_T_f_ratio = 1/6; 
boat.x_f_b_ratio = 1/8;     % Ratio between distance form bau to front foils, and boat length
boat.x_f_a_ratio = 1/8;     % Ratio between distance form stern to back foils, and boat length
boat.foil_leg_dx = 1*(boat.length/24);    % [m]
boat.foil_leg_dy = 0.3*(boat.length/24);   % Foil leg thickness [m]

% Foil data
foil.span = 5.6*(boat.length/24); % [m]
foil.cord_length = 1*(foil.span/5.6); % cord length [m]
foil.thickness = 0.2*foil.cord_length;
foil.time_const = 1/20; % Time const of foil dynamics [s]
foil.S = foil.span*foil.cord_length;
foil.C_L_0 = boat.mass*g/(water_density*foil.S*U_0^2);
foil.C_D_0 = 0;
foil.C_D_p = 0;     % Plastic drag
foil.C_M_0 = 0;
foil.AR = foil.span/foil.cord_length; % Aspect ratio
foil.C_L_alpha = (pi*foil.AR)/(1 + sqrt(1 + (foil.AR/2)^2)); % (p. 48 Beard & McLane)
foil.C_M_alpha = 0;
foil.k_C_D = 1/(pi*foil.AR);
foil.C_D_alpha = 2*foil.C_L_0*foil.C_L_alpha*foil.k_C_D;
foil.alpha_0 = foil.C_L_0/foil.C_L_alpha;
foil.sigma_M = 2;   % Transition rate between linear term and flat-plate term
foil.sigma_alpha_0 = 2*d2r*20;    % Cuttoff angle
foil.e = 1;
% For � teste "flyprinsipp"
foil.cfc = boat.mass*g/(1*water_density*foil.S*foil.C_L_alpha); % Const-force coefficient

% Rudder data
rudder.span = foil.span*0.2; % Height of each rudder as a function of foil span
rudder.cord_length = boat.foil_leg_dx*0.25;
rudder.thickness = boat.foil_leg_dy;
rudder.time_const = 1/20;
rudder.S = rudder.span*rudder.cord_length;
rudder.C_D_0 = 0;
rudder.AR = rudder.span/rudder.cord_length; % Aspect ratio
rudder.C_L_alpha = (pi*rudder.AR)/(1 + sqrt(1 + (rudder.AR/2)^2)); % (p. 48 Beard & McLane)
rudder.C_M_alpha = 0;

% Calculate width, height, foil depth and foil distances
boat.width = boat.length/boat.length_width_ratio;
boat.height = boat.width*boat.height_width_ratio;
boat.T_f = boat.T_f_ratio*boat.height;
boat.delta_T_f = boat.delta_T_f_ratio*boat.T_f;
boat.x_f_b = (1/2 - boat.x_f_b_ratio)*boat.length;
boat.x_f_a = (-1/2 + boat.x_f_a_ratio)*boat.length;

% Thruster data
propeller.z = boat.T_f; % Z position of thruster (body frame)
propeller.F_max = 40000; % Maximum force [N]
propeller.F_0 = 23742.4; % Initial force [N]

%%
% Divide main construction into blocks
% Center block:
B_c.length = boat.length;
B_c.height = (2/3)*boat.height;
B_c.width = (1/2)*boat.width;
B_c.volume = B_c.length*B_c.height*B_c.width;
% Two side blocks:
B_s.length = boat.length;
B_s.height = boat.height;
B_s.width = (1/4)*boat.width;
B_s.volume = B_s.length*B_s.height*B_s.width;

% Four foil legs (simplified)
B_fl.length = boat.foil_leg_dx;    % [m]
B_fl.width = boat.foil_leg_dy;   % [m]
B_fl.height = boat.T_f + boat.delta_T_f;
B_fl.volume = B_fl.length*B_fl.height*B_fl.width;

% Four foils (simplified as rectangular prisms)
B_f.length = foil.cord_length;    % [m]
B_f.width = foil.span;   % [m]
B_f.height = foil.thickness;
B_f.volume = B_f.length*B_f.height*B_f.width;

% Assuming homogeneous mass distribution
B_c.mass = boat.mass * (B_c.volume)/(B_c.volume + 2*B_s.volume + 4*B_fl.volume + 4*B_f.volume);
B_s.mass = boat.mass * (B_s.volume)/(B_c.volume + 2*B_s.volume + 4*B_fl.volume + 4*B_f.volume);
B_fl.mass = boat.mass * (B_fl.volume)/(B_c.volume + 2*B_s.volume + 4*B_fl.volume + 4*B_f.volume);
B_f.mass = boat.mass * (B_f.volume)/(B_c.volume + 2*B_s.volume + 4*B_fl.volume + 4*B_f.volume);

% Create block objects
block.center = RectangularCuboid(B_c.mass,B_c.length,B_c.width,B_c.height);
block.side = RectangularCuboid(B_s.mass,B_s.length,B_s.width,B_s.height);
block.foil_leg = RectangularCuboid(B_fl.mass,B_fl.length,B_fl.width,B_fl.height);
block.foil = RectangularCuboid(B_f.mass,B_f.length,B_f.width,B_f.height);

%% Create a block model of the vessel and add the blocks
block_model = VesselConstruction();
% Center block;
x = 0;
y = 0;
z = boat.height - B_c.height/2;
z = -z; % Reverse z to obtain the correct body frame
phi = 0;
theta = 0;
psi = 0;

block_model.addPart(ConstructionPart(block.center,x,y,z,phi,theta,psi));

% Left block;
x = 0;
y = -B_c.width/2 - B_s.width/2;
z = B_s.height/2;
z = -z; % Reverse z to obtain the correct body frame
phi = 0;
theta = 0;
psi = 0;

block_model.addPart(ConstructionPart(block.side,x,y,z,phi,theta,psi));

% Right block;
x = 0;
y = B_c.width/2 + B_s.width/2;
z = B_s.height/2;
z = -z; % Reverse z to obtain the correct body frame
phi = 0;
theta = 0;
psi = 0;

block_model.addPart(ConstructionPart(block.side,x,y,z,phi,theta,psi));

% Rear left foil leg
x = boat.x_f_a;
y = -foil.span/2 - B_fl.width/2;
z = -B_fl.height/2;
z = -z; % Reverse z to obtain the correct body frame
phi = 0;
theta = 0;
psi = 0;

block_model.addPart(ConstructionPart(block.foil_leg,x,y,z,phi,theta,psi));

% Rear right foil leg
x = boat.x_f_a;
y = foil.span/2 + B_fl.width/2;
z = -B_fl.height/2;
z = -z; % Reverse z to obtain the correct body frame
phi = 0;
theta = 0;
psi = 0;

block_model.addPart(ConstructionPart(block.foil_leg,x,y,z,phi,theta,psi));

% Front left foil leg
x = boat.x_f_b;
y = -foil.span/2 - B_fl.width/2;
z = -B_fl.height/2;
z = -z; % Reverse z to obtain the correct body frame
phi = 0;
theta = 0;
psi = 0;

block_model.addPart(ConstructionPart(block.foil_leg,x,y,z,phi,theta,psi));

% Front right foil leg
x = boat.x_f_b;
y = foil.span/2 + B_fl.width/2;
z = -B_fl.height/2;
z = -z; % Reverse z to obtain the correct body frame
phi = 0;
theta = 0;
psi = 0;

block_model.addPart(ConstructionPart(block.foil_leg,x,y,z,phi,theta,psi));

% Rear upper foil
x = boat.x_f_a;
y = 0;
z = -B_fl.height + 2*boat.delta_T_f + foil.thickness/2;
z = -z; % Reverse z to obtain the correct body frame
phi = 0;
theta = 0;
psi = 0;

block_model.addPart(ConstructionPart(block.foil,x,y,z,phi,theta,psi));

% Rear lower foil
x = boat.x_f_a;
y = 0;
z = -B_fl.height + foil.thickness/2;
z = -z; % Reverse z to obtain the correct body frame
phi = 0;
theta = 0;
psi = 0;

block_model.addPart(ConstructionPart(block.foil,x,y,z,phi,theta,psi));

% Front upper foil
x = boat.x_f_b;
y = 0;
z = -B_fl.height + 2*boat.delta_T_f + foil.thickness/2;
z = -z; % Reverse z to obtain the correct body frame
phi = 0;
theta = 0;
psi = 0;

block_model.addPart(ConstructionPart(block.foil,x,y,z,phi,theta,psi));

% Front lower foil
x = boat.x_f_b;
y = 0;
z = -B_fl.height + foil.thickness/2;
z = -z; % Reverse z to obtain the correct body frame
phi = 0;
theta = 0;
psi = 0;

block_model.addPart(ConstructionPart(block.foil,x,y,z,phi,theta,psi));

%% Simulink input data

r_g = [block_model.CG_x, block_model.CG_y, block_model.CG_z]';
% r_g = [0,0,-3]';
I_b = block_model.I;
mass = block_model.mass;
eta_0 = [0,0,z_d, 0*d2r,0,0]';
nu_0 = [U_0,0,0, 0,0,0]';
tau_0 = [0,0,0, 0,0,0]';

% Attack points: Foils
force_attack_point.foil.r_FL = [boat.x_f_b,-foil.span/4,boat.T_f]';
force_attack_point.foil.r_FR = [boat.x_f_b,foil.span/4,boat.T_f]';
force_attack_point.foil.r_AL = [boat.x_f_a,-foil.span/4,boat.T_f]';
force_attack_point.foil.r_AR = [boat.x_f_a,foil.span/4,boat.T_f]';

% Attack points: Rudders
force_attack_point.rudder.r_FL = [boat.x_f_b-boat.foil_leg_dx/2+rudder.cord_length/2,-foil.span/2,boat.T_f-rudder.span/2]';
force_attack_point.rudder.r_FR = [boat.x_f_b-boat.foil_leg_dx/2+rudder.cord_length/2,foil.span/2,boat.T_f-rudder.span/2]';
force_attack_point.rudder.r_AL = [boat.x_f_a-boat.foil_leg_dx/2+rudder.cord_length/2,-foil.span/2,boat.T_f-rudder.span/2]';
force_attack_point.rudder.r_AR = [boat.x_f_a-boat.foil_leg_dx/2+rudder.cord_length/2,foil.span/2,boat.T_f-rudder.span/2]';

% State saturation
upper_saturation = [inf, inf, 100, 90*d2r, 90*d2r, inf];
lower_saturation = [-inf, -inf, -(boat.T_f-boat.delta_T_f)*10, -90*d2r, -90*d2r, -inf];

% upper_saturation = [inf,inf,inf,inf,inf,inf];
% lower_saturation = -[inf,inf,inf,inf,inf,inf];
path.R = 200;
path.V_g = 10;

% Wave model

% wave.regular = 0;
% wave.longcrested = 0;
% wave.direction = 0;

noise.on = true;
noise.power = 1e-7;
noise.L = [1 d2r/abs(force_attack_point.foil.r_FL(2)), d2r/abs(force_attack_point.foil.r_FL(1)), 1 d2r/abs(force_attack_point.foil.r_FL(2)), d2r/abs(force_attack_point.foil.r_FL(1))]';

wind = 0*[0,1,0,-0.5,0,0]'*mass*g/200;

% Wave data set: amplitude, frequency, phase angle, direction, energy
% density
wave.dataset = [1*0.7, 0.86, 0, 180*d2r, 0  % For U = 10 m/s: wp = 0.86 rad/s, zeta_a = 0.7 m
    0*1, 2, 0, 135*d2r, 0
    0, 0.5, 0, -0*d2r, 0];

% Wave data from PM spectrum
U_wind = 10; % [m/s]
theta_0 = 0*d2r; % [rad]
N = 20; % # Frequency components , 20
K = 5; % # Directional components , 5
phaseOn = 1;
wave.dataset = getWaveDataset( U_wind, theta_0, N, K, phaseOn);
%                 0.5, 1, rand*2*pi, -10*d2r
%                 0.5, 1, rand*2*pi, 10*d2r
%                 0.5, 1, rand*2*pi, 15*d2r
%                 0.5, 1, rand*2*pi, -8*d2r
%                 0.5, 1, rand*2*pi, 20*d2r
%                 0.5, 1, rand*2*pi, -30*d2r];

%% Controller gains

% r
controller.r.K_p = 1.333;
controller.r.K_i = 0.1;
controller.r.K_d = 1.1966;

% v
controller.v.K_p = 0.0721;
controller.v.K_i = 0.0255;
controller.v.K_d = 0;

% Surge
controller.u.K_p = 2.333;
controller.u.K_i = 0.1;
controller.u.K_d = 2;

